% 
% Simple class for presentation slides using the Beamer class.
% icsslides.sty, Version 0.13, 2008/05/15
%
% For the documentation of Beamer see:
% http://www.ctan.org/tex-archive/macros/latex/contrib/beamer/doc/beameruserguide.pdf
% 
% Right now this class does not do much more than setting colours and
% loading the ICS logo. If you wish to have more information in the frame
% header and footer, consider changing the \useoutertheme setting (see
% Beamer documentation).
%
% Please send complaints and other requests to Andre Schumacher
% (schumach@tcs.hut.fi).

% ----------------------------------------------------------
%
% Changelog
%
% Version 0.13: Theorem body text now in italics, frame number in
%               frame footer, reorganised stuff #andre
% Version 0.12: Changed block templates and colors #andre

\NeedsTeXFormat{LaTeX2e}
\ProvidesClass{icsslides}[2008/05/15 Slides Template for ICS (uses Beamer)]

\RequirePackage{color}
\LoadClass{beamer}

% ----------------------------------------------------------

% SETTINGS

% This setting determines the information displayed in frame header and footer.
% If you prefer a smaller header and more content in the footer, try
% commenting the next line und uncommenting the following line.
% Note: if you prefer the infolines styles you might also want to comment out
% the changes made to the footer further below.
\useoutertheme{split}
%\useoutertheme{infolines}

% This causes later content on overlay slides to appear transparent.
\setbeamercovered{transparent}

% Set list bullet symbol and block style:
\setbeamertemplate{itemize items}[ball]
\setbeamertemplate{blocks}[rounded][shadow=true]

% Set logo:
\logo{\includegraphics[height=2cm]{icslogo.pdf}\vspace*{-0.5cm}\hspace*{0.55\paperwidth}}

% Contact information:
\institute[ICS]{Department of Information and Computer Science\\
Helsinki University of Technology}

% ----------------------------------------------------------

% FONTS

% Choose default TKK fonts:
\renewcommand{\sfdefault}{pbe}
\renewcommand{\rmdefault}{lea}

% This is somewhat of a workaround as the TKK fonts do not seem
% to have an italics shape and text in the theorem body looks
% nicer in italics.
\setbeamerfont{theorem body}{family=\sffamily,shape=\itshape,family*={ptm}}

% Modified from beamer/base/beamerbaseauxtemplates.sty
\setbeamertemplate{theorem begin}
{
  \normalfont
  \begin{\inserttheoremblockenv}
  {%
    \inserttheoremname
    \ifx\inserttheoremaddition\@empty\else\ (\inserttheoremaddition)\fi%
  }%
  \usebeamerfont{theorem body}
}

% ----------------------------------------------------------

% COLORS

% Define colors:
\definecolor{tkkgray}{rgb}{0.6,0.6,0.6}
\definecolor{tkkblue}{rgb}{0,0,0.8}
\definecolor{blockbodyc}{rgb}{0.9,0.9,1.0}
\definecolor{alertblockbodyc}{rgb}{1.0,0.9,0.9}
\definecolor{exampleblockbodyc}{rgb}{0.9,1.0,0.9}
\definecolor{darkgreen}{rgb}{0.0,0.8,0.0}

% Set colors used for headers and footers (see infolines theme):
\setbeamercolor{palette primary}{fg=black, bg=tkkgray}
\setbeamercolor{palette secondary}{fg=tkkblue, bg=white}
\setbeamercolor{palette tertiary}{fg=tkkgray, bg=tkkblue}

% Set specific colors:
\setbeamercolor{titlelike}{fg=tkkblue}
\setbeamercolor{section in toc}{fg=tkkblue}
\setbeamercolor{itemize subitem}{fg=tkkblue}
\setbeamercolor{itemize item}{fg=tkkblue}
\setbeamercolor{enumerate item}{fg=tkkblue}
\setbeamercolor{block title}{fg=white,bg=tkkblue}
\setbeamercolor{block body}{bg=blockbodyc}
\setbeamercolor{block title alerted}{fg=white,bg=red}
\setbeamercolor{block body alerted}{bg=alertblockbodyc}
\setbeamercolor{block title example}{fg=white,bg=darkgreen}
\setbeamercolor{block body example}{bg=exampleblockbodyc}
\setbeamercolor{section in head/foot}{fg=white, bg=tkkblue}

% ----------------------------------------------------------

% HEADER & FOOTER

% Use this command to define own slide header.
%\setbeamertemplate{headline}
%{%
% \begin{beamercolorbox}{titlelike}
%   \vskip2pt\insertnavigation{\paperwidth}\vskip2pt
%   \end{beamercolorbox}%
%\rule{\paperwidth}{0.5mm}%
%}

% Modified from beamer/themes/outer/beamerouterthemesplit.sty
% Adds page number (actually frame number, which differs in the
% case of overlays)
\setbeamertemplate{footline}
{%
  \leavevmode%
  \hbox{\begin{beamercolorbox}[wd=.5\paperwidth,ht=2.5ex,dp=1.125ex,leftskip=.3cm plus1fill,rightskip=.3cm]{author in head/foot}%
    \usebeamerfont{author in head/foot}\insertshortauthor
  \end{beamercolorbox}%
  \begin{beamercolorbox}[wd=.5\paperwidth,ht=2.5ex,dp=1.125ex,leftskip=.3cm,rightskip=.3cm plus1fil]{title in head/foot}%
    \usebeamerfont{title in head/foot}\insertshorttitle\hfill\insertframenumber/\inserttotalframenumber
  \end{beamercolorbox}}%
  \vskip0pt%
}

% ----------------------------------------------------------

